==> nui_core-0.0.3.tar.gz: lib/ui/component/nui_app.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/config/nui_app_config.dart';
import 'package:nui_core/config/nui_global_config.dart';
import 'package:nui_core/nui_core.dart';
import '../../nui_core.dart';
import '../nui_display_util.dart';

typedef AsyncFunction = Future<dynamic> Function(BuildContext context);
typedef OpenNewScreenFunction = void Function(Widget page);
typedef ThemeInflaterFunction = ThemeData Function(BuildContext context);
typedef OverlayInflaterFunction = NUIOverlayStyle Function(BuildContext context);
typedef AppNameByBuildType = String Function(BuildType buildType);
typedef NUIAppCrashHandler = void Function(Object error, StackTrace stackTrace);

enum BuildType{
  DEV,
  UAT,
  BETA,
  PRE_PROD,
  PROD,
  OTH1,
  OTH2,
  OTH3,
  OTH4,
  OTH5
}

BuildType buildTypeValueOf(String value){
  switch(value){
    case "DEV": return BuildType.DEV;
    case "UAT": return BuildType.UAT;
    case "BETA": return BuildType.BETA;
    case "PREPROD":
    case "PRE-PROD":
    case "PRE_PROD": return BuildType.PRE_PROD;
    case "PROD": return BuildType.PROD;
    case "OTH1": return BuildType.OTH1;
    case "OTH2": return BuildType.OTH2;
    case "OTH3": return BuildType.OTH3;
    case "OTH4": return BuildType.OTH4;
    case "OTH5": return BuildType.OTH5;
    default: return BuildType.DEV;
  }
}

extension BuildTypeExt on BuildType{
  String value(){
    switch(this){
      case BuildType.DEV : return "DEV";
      case BuildType.UAT : return "UAT";
      case BuildType.BETA : return "BETA";
      case BuildType.PRE_PROD : return "PRE_PROD";
      case BuildType.PROD : return "PROD";
      case BuildType.OTH1 : return "OTH1";
      case BuildType.OTH2 : return "OTH2";
      case BuildType.OTH3 : return "OTH3";
      case BuildType.OTH4 : return "OTH4";
      case BuildType.OTH5 : return "OTH5";
      default: return "DEV";
    }
  }

  String configFile(){
    switch(this){
      case BuildType.DEV : return "dev_config";
      case BuildType.UAT : return "uat_config";
      case BuildType.BETA : return "beta_config";
      case BuildType.PRE_PROD : return "pre_prod_config";
      case BuildType.PROD : return "prod_config";
      case BuildType.OTH1 : return "oth_config";
      case BuildType.OTH2 : return "oth2_config";
      case BuildType.OTH3 : return "oth3_config";
      case BuildType.OTH4 : return "oth4_config";
      case BuildType.OTH5 : return "oth5_config";
      default: return "dev_config";
    }
  }

  NUIAppConfig getConfig() {
     return NUIGlobalConfig.get(module: this.value()).getConfig();
  }

  Map<String, dynamic> getRawConfig() {
    return NUIGlobalConfig.get(module: this.value()).getAll();
  }
}

void runNUIApp(NUIApp app, {NUIAppCrashHandler crashHandler}){
  runZonedGuarded<Future<void>>(() async {
    runApp(app);
  }, (Object error, StackTrace stackTrace) {
    if(crashHandler != null) crashHandler(error, stackTrace);
  });
  FlutterError.onError = (FlutterErrorDetails details){
    Zone.current.handleUncaughtError(details.exception, details.stack);
  };
}

class NUISnackItem{
  NUISnackItem({
    @required this.content,
    this.action,
    this.duration,
    this.backgroundColor,
    this.elevation});
  Widget content;
  SnackBarAction action;
  Duration duration;
  Color backgroundColor;
  double elevation;
}

void snack({@required BuildContext context, @required Widget content, SnackBarAction action, Duration duration, Color backgroundColor, double elevation}){
  NUIApp.snack(context: context, content: content, action: action, duration: duration, backgroundColor: backgroundColor, elevation: elevation);
}

void snackByItem({@required BuildContext context, @required NUISnackItem item}){
  NUIApp.snackByItem(context: context, item: item);
}

NUIApp getNUIApp(){
  return NUIApp._instance;
}

String getString(String key, {List<dynamic> params}) => NUIGlobalConfig.get(module: NUIApp._STRING_CONFIGS).getString(key, params: params);
NUIAppConfig appConfig() => getNUIApp().appConfig();
T configField<T>(String key) => getNUIApp().buildType.getRawConfig()[key] as T;

class NUIApp extends StatelessWidget {
  static const String _STRING_CONFIGS = "NUIAppStringConfigs";
  static const String _STRING_CONFIG_FILE = "strings.json";
  static OpenNewScreenFunction _navigator;
  static BuildType _appBuildType;
  static BuildType appBuild(){
    return _appBuildType;
  }

  NUIApp({@required this.buildType, @required this.title, @required this.homeScreen, this.routes, this.initialRoute, @required this.themeData, @required this.overlayStyle, @required this.loadingScreen, this.preAppTask}){
    _appBuildType = buildType;
  }

  static NUIApp _instance;

  final NUIRoute loadingScreen;
  final NUIRoute homeScreen;
  final OverlayInflaterFunction overlayStyle;
  final ThemeInflaterFunction themeData;
  final String title;
  final AsyncFunction preAppTask;
  final Map<String, NUIRoute> routes;
  final String initialRoute;
  final BuildType buildType;

  List<AsyncVoidTask> _addOnTasks = List();

  static NUIApp get(){
    return _instance;
  }

  void addNewTask(AsyncVoidTask task) => _addOnTasks.add(task);

  T configField<T>(String key) => buildType.getRawConfig()[key] as T;

  NUIAppConfig appConfig() => buildType.getConfig();

  String getString(String key, {List<dynamic> params}) => NUIGlobalConfig.get(module: _STRING_CONFIGS).getString(key, params: params);

  static void push(Widget page){
    if(_navigator != null){
      _navigator(page);
    }
  }

  static void snackByItem({@required BuildContext context, @required NUISnackItem item}){
    return snack(context: context, content: item.content, action: item.action, duration: item.duration, backgroundColor: item.backgroundColor, elevation: item.elevation);
  }

  static void snack({@required BuildContext context, @required Widget content, SnackBarAction action, Duration duration, Color backgroundColor, double elevation}){
    logNUI("NUIApp", "Snack with duration : ${duration?.inSeconds}");
    final snackBar = SnackBar(
      backgroundColor: backgroundColor,
      content: content,
      action: action,
      elevation: elevation ?? 3,
      duration: duration ?? Duration(seconds: 3),
    );
    try{
      Scaffold.of(context).showSnackBar(snackBar);
    } catch(e){
      log("NUIApp -> Failed to show snack bar with exception : " + e.toString());
    }
  }

  Future<dynamic> preTask(BuildContext context) async{
    NUIBusEventBuilder().build();
    final config1 = NUIGlobalConfigBuilder().module(_STRING_CONFIGS).fileName(_STRING_CONFIG_FILE).build();
    final config2 = NUIGlobalConfigBuilder().module(_appBuildType.value()).fileName(_appBuildType.configFile()).build();

    if(!_addOnTasks.isNullOrEmpty()){
      _addOnTasks.forEach((element) async{
        await element();
      });
    }

    if(preAppTask != null) await preAppTask(context);
    await config1;
    await config2;
    return true;
  }

  @override
  Widget build(BuildContext context) {
    NUIDisplayUtil.setStatusBarStyleAttr(overlayStyle(context));
    NUIBusEventBuilder().build();
    NUIGlobalConfigBuilder().module(_STRING_CONFIGS).fileName(_STRING_CONFIG_FILE).build();
    NUIGlobalConfigBuilder().module(_appBuildType.value()).fileName(_appBuildType.configFile()).build();
    _instance = this;

    print("Config -> ${jsonEncode(getNUIApp().appConfig())}");

    _navigator = (page){
      NUINavigator.push(context, page);
    };

    Widget startingPage = NUIInitialScreen(homeScreen: homeScreen, loadingScreen: loadingScreen, preAppTask: preTask);

    if(routes == null){
      return MaterialApp(
          title: title,
          initialRoute: initialRoute,
          theme: themeData(context),
          home: startingPage
      );
    }
    else{
      return MaterialApp(
          title: title,
          initialRoute: initialRoute,
          theme: themeData(context),
          routes: routes ?? null,
          home: startingPage
      );
    }
  }
}

class NUIInitialScreen extends StatefulWidget {
  NUIInitialScreen({@required this.homeScreen, @required this.loadingScreen, this.routes, this.preAppTask});
  final NUIRoute loadingScreen;
  final NUIRoute homeScreen;
  final AsyncFunction preAppTask;
  final Map<String, NUIRoute> routes;

  @override
  _NUIInitialScreenState createState() => _NUIInitialScreenState();
}

class _NUIInitialScreenState extends State<NUIInitialScreen> {
  Future<dynamic> preAppTaskResult;

  @override
  void initState(){
    super.initState();
    preAppTaskResult = widget.preAppTask(context);
  }

  @override
  Widget build(BuildContext context) {
    return NUIFutureWidget(
        futureValue: preAppTaskResult,
        callback: (state, data, message) {
          if(state == FutureState.SUCCESS){
            NUIAsync.delay(Duration(milliseconds: 500), () {
              NUINavigator.popAndPush(context, widget.homeScreen(context), transition: PageTransition.FADE, duration: Duration(milliseconds: 500));
            });
          }
          return widget.loadingScreen(context);
        });
  }
}

==> nui_core-0.0.7.tar.gz: lib/ui/component/nui_app.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/config/nui_app_config.dart';
import 'package:nui_core/config/nui_global_config.dart';
import 'package:nui_core/nui_core.dart';
import '../../nui_core.dart';
import '../nui_display_util.dart';

typedef AsyncFunction = Future<dynamic> Function(BuildContext context);
typedef OpenNewScreenFunction = void Function(Widget page);
typedef ThemeInflaterFunction = ThemeData Function(BuildContext context);
typedef OverlayInflaterFunction = NUIOverlayStyle Function(BuildContext context);
typedef AppNameByBuildType = String Function(BuildType buildType);
typedef NUIAppCrashHandler = void Function(Object error, StackTrace stackTrace);

enum BuildType{
  DEV,
  UAT,
  BETA,
  PRE_PROD,
  PROD,
  OTH1,
  OTH2,
  OTH3,
  OTH4,
  OTH5
}

BuildType buildTypeValueOf(String value){
  switch(value){
    case "DEV": return BuildType.DEV;
    case "UAT": return BuildType.UAT;
    case "BETA": return BuildType.BETA;
    case "PREPROD":
    case "PRE-PROD":
    case "PRE_PROD": return BuildType.PRE_PROD;
    case "PROD": return BuildType.PROD;
    case "OTH1": return BuildType.OTH1;
    case "OTH2": return BuildType.OTH2;
    case "OTH3": return BuildType.OTH3;
    case "OTH4": return BuildType.OTH4;
    case "OTH5": return BuildType.OTH5;
    default: return BuildType.DEV;
  }
}

extension BuildTypeExt on BuildType{
  String value(){
    switch(this){
      case BuildType.DEV : return "DEV";
      case BuildType.UAT : return "UAT";
      case BuildType.BETA : return "BETA";
      case BuildType.PRE_PROD : return "PRE_PROD";
      case BuildType.PROD : return "PROD";
      case BuildType.OTH1 : return "OTH1";
      case BuildType.OTH2 : return "OTH2";
      case BuildType.OTH3 : return "OTH3";
      case BuildType.OTH4 : return "OTH4";
      case BuildType.OTH5 : return "OTH5";
      default: return "DEV";
    }
  }

  String configFile(){
    switch(this){
      case BuildType.DEV : return "dev_config";
      case BuildType.UAT : return "uat_config";
      case BuildType.BETA : return "beta_config";
      case BuildType.PRE_PROD : return "pre_prod_config";
      case BuildType.PROD : return "prod_config";
      case BuildType.OTH1 : return "oth_config";
      case BuildType.OTH2 : return "oth2_config";
      case BuildType.OTH3 : return "oth3_config";
      case BuildType.OTH4 : return "oth4_config";
      case BuildType.OTH5 : return "oth5_config";
      default: return "dev_config";
    }
  }

  NUIAppConfig getConfig() {
     return NUIGlobalConfig.get(module: this.value()).getConfig();
  }

  Map<String, dynamic> getRawConfig() {
    return NUIGlobalConfig.get(module: this.value()).getAll();
  }
}

void runNUIApp(NUIApp app, {NUIAppCrashHandler crashHandler}){
  runZonedGuarded<Future<void>>(() async {
    runApp(app);
  }, (Object error, StackTrace stackTrace) {
    if(crashHandler != null) crashHandler(error, stackTrace);
  });
  FlutterError.onError = (FlutterErrorDetails details){
    Zone.current.handleUncaughtError(details.exception, details.stack);
  };
}

class NUISnackItem{
  NUISnackItem({
    @required this.content,
    this.action,
    this.duration,
    this.backgroundColor,
    this.elevation});
  Widget content;
  SnackBarAction action;
  Duration duration;
  Color backgroundColor;
  double elevation;
}

void snack({@required BuildContext context, @required Widget content, SnackBarAction action, Duration duration, Color backgroundColor, double elevation}){
  NUIApp.snack(context: context, content: content, action: action, duration: duration, backgroundColor: backgroundColor, elevation: elevation);
}

void snackByItem({@required BuildContext context, @required NUISnackItem item}){
  NUIApp.snackByItem(context: context, item: item);
}

NUIApp getNUIApp(){
  return NUIApp._instance;
}

String getString(String key, {List<dynamic> params}) => NUIGlobalConfig.get(module: NUIApp._STRING_CONFIGS).getString(key, params: params);
NUIAppConfig appConfig() => getNUIApp().appConfig();
T configField<T>(String key) => getNUIApp().buildType.getRawConfig()[key] as T;

class NUIApp extends StatelessWidget {
  static const String _STRING_CONFIGS = "NUIAppStringConfigs";
  static const String _STRING_CONFIG_FILE = "strings.json";
  static OpenNewScreenFunction _navigator;
  static BuildType _appBuildType;
  static BuildType appBuild(){
    return _appBuildType;
  }

  NUIApp({@required this.buildType, @required this.title, @required this.homeScreen, this.routes, this.initialRoute, @required this.themeData, @required this.overlayStyle, @required this.loadingScreen, this.preAppTask}){
    _appBuildType = buildType;
  }

  static NUIApp _instance;

  final NUIRoute loadingScreen;
  final NUIRoute homeScreen;
  final OverlayInflaterFunction overlayStyle;
  final ThemeInflaterFunction themeData;
  final String title;
  final AsyncFunction preAppTask;
  final Map<String, NUIRoute> routes;
  final String initialRoute;
  final BuildType buildType;

  List<AsyncVoidTask> _addOnTasks = List();

  static NUIApp get(){
    return _instance;
  }

  void addNewTask(AsyncVoidTask task) => _addOnTasks.add(task);

  T configField<T>(String key) => buildType.getRawConfig()[key] as T;

  NUIAppConfig appConfig() => buildType.getConfig();

  String getString(String key, {List<dynamic> params}) => NUIGlobalConfig.get(module: _STRING_CONFIGS).getString(key, params: params);

  static void push(Widget page){
    if(_navigator != null){
      _navigator(page);
    }
  }

  static void snackByItem({@required BuildContext context, @required NUISnackItem item}){
    return snack(context: context, content: item.content, action: item.action, duration: item.duration, backgroundColor: item.backgroundColor, elevation: item.elevation);
  }

  static void snack({@required BuildContext context, @required Widget content, SnackBarAction action, Duration duration, Color backgroundColor, double elevation}){
    logNUI("NUIApp", "Snack with duration : ${duration?.inSeconds}");
    final snackBar = SnackBar(
      backgroundColor: backgroundColor,
      content: content,
      action: action,
      elevation: elevation ?? 3,
      duration: duration ?? Duration(seconds: 3),
    );
    try{
      Scaffold.of(context).showSnackBar(snackBar);
    } catch(e){
      log("NUIApp -> Failed to show snack bar with exception : " + e.toString());
    }
  }

  Future<dynamic> preTask(BuildContext context) async{
    NUIBusEventBuilder().build();
    final config1 = NUIGlobalConfigBuilder().module(_STRING_CONFIGS).fileName(_STRING_CONFIG_FILE).build();
    final config2 = NUIGlobalConfigBuilder().module(_appBuildType.value()).fileName(_appBuildType.configFile()).build();

    if(!_addOnTasks.isNullOrEmpty()){
      _addOnTasks.forEach((element) async{
        await element();
      });
    }

    await config1;
    await config2;
    if(preAppTask != null) await preAppTask(context);
    return true;
  }

  @override
  Widget build(BuildContext context) {
    NUIDisplayUtil.setStatusBarStyleAttr(overlayStyle(context));
    _instance = this;
    _navigator = (page){
      NUINavigator.push(context, page);
    };

//    addNewTask(() async{
//      NUIBusEventBuilder().build();
//      final config1 = NUIGlobalConfigBuilder().module(_STRING_CONFIGS).fileName(_STRING_CONFIG_FILE).build();
//      final config2 = NUIGlobalConfigBuilder().module(_appBuildType.value()).fileName(_appBuildType.configFile()).build();
//      print("Config -> ${jsonEncode(getNUIApp().appConfig())}");
//    });

    Widget startingPage = NUIInitialScreen(homeScreen: homeScreen, loadingScreen: loadingScreen, preAppTask: preTask);

    if(routes == null){
      return MaterialApp(
          title: title,
          initialRoute: initialRoute,
          theme: themeData(context),
          home: startingPage
      );
    }
    else{
      return MaterialApp(
          title: title,
          initialRoute: initialRoute,
          theme: themeData(context),
          routes: routes ?? null,
          home: startingPage
      );
    }
  }
}

class NUIInitialScreen extends StatefulWidget {
  NUIInitialScreen({@required this.homeScreen, @required this.loadingScreen, this.routes, this.preAppTask});
  final NUIRoute loadingScreen;
  final NUIRoute homeScreen;
  final AsyncFunction preAppTask;
  final Map<String, NUIRoute> routes;

  @override
  _NUIInitialScreenState createState() => _NUIInitialScreenState();
}

class _NUIInitialScreenState extends State<NUIInitialScreen> {
  Future<dynamic> preAppTaskResult;

  @override
  void initState(){
    super.initState();
    preAppTaskResult = widget.preAppTask(context);
  }

  @override
  Widget build(BuildContext context) {
    return NUIFutureWidget(
        futureValue: preAppTaskResult,
        callback: (state, data, message) {
          if(state == FutureState.SUCCESS){
            NUIAsync.delay(Duration(milliseconds: 500), () {
              NUINavigator.popAndPush(context, widget.homeScreen(context), transition: PageTransition.FADE, duration: Duration(milliseconds: 500));
            });
          }
          return widget.loadingScreen(context);
        });
  }
}
